---------------------------------------------------------------------------
-- This VHDL file was developed by Daniel Llamocca.  It may be
-- freely copied and/or distributed at no cost.  Any persons using this
-- file for any purpose do so at their own risk, and are responsible for
-- the results of such use.  Daniel Llamocca does not guarantee that
-- this file is complete, correct, or fit for any particular purpose.
-- NO WARRANTY OF ANY KIND IS EXPRESSED OR IMPLIED.  This notice must
-- accompany any copy of this file.
--------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
use ieee.std_logic_arith.all;
 
-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--USE ieee.numeric_std.ALL;
 
ENTITY atb_sqrt_nr IS
	generic (N: INTEGER:=12;
	         x: INTEGER:= 4);
END atb_sqrt_nr;
 
ARCHITECTURE behavior OF atb_sqrt_nr IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
    COMPONENT sqrt_nr
    PORT(
         clock : IN  std_logic;
         resetn : IN  std_logic;
         s : IN  std_logic;
         D : IN  std_logic_vector(N-1 downto 0);
         Q : OUT  std_logic_vector(N/2 + x -1 downto 0);
         done : OUT  std_logic
        );
    END COMPONENT;
    
   constant NQ: INTEGER := N/2 + x;
   --Inputs
   signal clock : std_logic := '0';
   signal resetn : std_logic := '0';
   signal s : std_logic := '0';
   signal D : std_logic_vector(N-1 downto 0) := (others => '0');

 	--Outputs
   signal Q : std_logic_vector(NQ-1 downto 0);
   signal done : std_logic;

   -- Clock period definitions
   constant PERIOD : time := 20 ns;
	constant DUTY_CYCLE: real := 0.5;
	constant OFFSET: time := 100 ns;
 
BEGIN
 
	-- Instantiate the Unit Under Test (UUT)
   uut: sqrt_nr PORT MAP (clock => clock, resetn => resetn, s => s, D => D, Q => Q, done => done);

-- Clock Process Definition
ck: process
		begin
			wait for OFFSET; -- Wait 100 ns for global reset to finish
			clock_loop: loop -- loop that defines the PLB_clock
				clock <= '0';
				wait for (PERIOD - (PERIOD*DUTY_CYCLE));
				clock <= '1';
				wait for (PERIOD*DUTY_CYCLE);
			end loop clock_loop;
	 end process;

   -- Stimulus process
   stim_proc: process
   begin		
      -- hold reset state for 100 ns.
      wait for OFFSET;	
		resetn <= '1';
      wait for PERIOD;
      -- insert stimulus here 
		s <= '1'; D <= conv_std_logic_vector(40,N); wait for PERIOD;
		s <= '1'; D <= conv_std_logic_vector(16,N); wait for PERIOD;
		s <= '1'; D <= conv_std_logic_vector(32,N); wait for PERIOD;
		s <= '1'; D <= conv_std_logic_vector(4,N); wait for PERIOD;
		s <= '1'; D <= conv_std_logic_vector(9,N); wait for PERIOD;
		s <= '1'; D <= conv_std_logic_vector(25,N); wait for PERIOD;
		s <= '1'; D <= conv_std_logic_vector(36,N); wait for PERIOD;
		s <= '1'; D <= conv_std_logic_vector(49,N); wait for 6*PERIOD;
		s <= '0';
		
      wait;
   end process;

END;
