---------------------------------------------------------------------------
-- This VHDL file was developed by Daniel Llamocca based on the ALTERA 
-- lpm_add_sub megafunction VHDL description.  It may be
-- freely copied and/or distributed at no cost.  Any persons using this
-- file for any purpose do so at their own risk, and are responsible for
-- the results of such use.  Daniel Llamocca does not guarantee that
-- this file is complete, correct, or fit for any particular purpose.
-- NO WARRANTY OF ANY KIND IS EXPRESSED OR IMPLIED.  This notice must
-- accompany any copy of this file.
--------------------------------------------------------------------------

-- LIBRARY USED----------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

entity daniel_ADD_SUB_UNSIGNED is
    generic (
        lpm_width      : natural;    -- MUST be greater than 0
        lpm_direction  : string  := "UNUSED";
		  cin_used		  : string := "YES"
    );
    
    port (
        dataa   : in std_logic_vector(lpm_width-1 downto 0);
        datab   : in std_logic_vector(lpm_width-1 downto 0);
        cin     : in std_logic := '0';
        add_sub : in std_logic := '1';
        result   : out std_logic_vector(lpm_width-1 downto 0)
    );
end daniel_ADD_SUB_UNSIGNED;

architecture structure of daniel_ADD_SUB_UNSIGNED is
begin

a0: assert (cin_used = "NO" or cin_used = "YES")
	    report "You have to set cin_used to either YES or NO!!!"
		 severity error;
		
a1: if cin_used = "NO" generate
		process (dataa,datab, cin, add_sub)
		begin
			if ((lpm_direction = "ADD") or ((lpm_direction = "UNUSED") and (add_sub = '1'))) then
					result <= dataa + datab;	
         elsif ((lpm_direction = "SUB") or ((lpm_direction = "UNUSED") and (add_sub = '0'))) then					
					result <= dataa - datab;
			elsif (lpm_direction /= "UNUSED") then
            ASSERT FALSE
            REPORT "Illegal lpm_direction property value for LPM_ADD_SUB!"
            SEVERITY ERROR;
        end if;
		end process;		
	end generate;
	
a2: if cin_used = "YES" generate
		process (dataa,datab, cin, add_sub)
		begin
			if ((lpm_direction = "ADD") or ((lpm_direction = "UNUSED") and (add_sub = '1'))) then
					result <= dataa + datab + cin;	
         elsif ((lpm_direction = "SUB") or ((lpm_direction = "UNUSED") and (add_sub = '0'))) then					
					result <= dataa - datab + cin - 1;
			elsif (lpm_direction /= "UNUSED") then
            ASSERT FALSE
            REPORT "Illegal lpm_direction property value for LPM_ADD_SUB!"
            SEVERITY ERROR;
        end if;
		end process;		
	end generate;	


end structure;