---------------------------------------------------------------------------
-- This VHDL file was developed by Daniel Llamocca based on the ALTERA 
-- lpm_add_sub megafunction VHDL description.  It may be
-- freely copied and/or distributed at no cost.  Any persons using this
-- file for any purpose do so at their own risk, and are responsible for
-- the results of such use.  Daniel Llamocca does not guarantee that
-- this file is complete, correct, or fit for any particular purpose.
-- NO WARRANTY OF ANY KIND IS EXPRESSED OR IMPLIED.  This notice must
-- accompany any copy of this file.
--------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;

-- ENTITY DECLARATION
entity daniel_ADD_SUB is
    generic (
        lpm_width          : natural;    -- MUST be greater than 0
        lpm_representation : string := "SIGNED";
        lpm_direction      : string := "UNUSED"; -- UNUSED, ADD, SUB
		  cin_used				: string := "YES" -- "YES", "NO"
    );
    
    port (
        dataa   : in std_logic_vector(lpm_width-1 downto 0);
        datab   : in std_logic_vector(lpm_width-1 downto 0);
		  cin     : in std_logic := '0';		  
        add_sub : in std_logic := '1'; -- 1: add, 0: sub
        result   : out std_logic_vector(lpm_width-1 downto 0)        
    );
end daniel_ADD_SUB;
-- END OF ENTITY

-- BEGINNING OF ARCHITECTURE
architecture structure of daniel_ADD_SUB is

-- COMPONENT DECLARATION
    component daniel_ADD_SUB_SIGNED
            generic (
                lpm_width     : natural;    -- MUST be greater than 0
                lpm_direction : string := "UNUSED" ;
					 cin_used      : string := "YES"
            );
            port (
                dataa   : in std_logic_vector(lpm_width downto 1);
                datab   : in std_logic_vector(lpm_width downto 1);
					 cin     : in std_logic := '0';
                add_sub : in std_logic := '1';
                result   : out std_logic_vector(lpm_width-1 downto 0)
            );
    end component;

    component daniel_ADD_SUB_UNSIGNED
            generic (
                lpm_width     : natural;    -- MUST be greater than 0
                lpm_direction : string := "UNUSED";                
					 cin_used      : string := "YES"
            );
            port (
                dataa   : in std_logic_vector(lpm_width-1 downto 0);
                datab   : in std_logic_vector(lpm_width-1 downto 0);
					 cin     : in std_logic := '0';
                add_sub : in std_logic := '1';
                result   : out std_logic_vector(lpm_width-1 downto 0)
            );
    end component;

begin

    L1: if (LPM_representation = "UNSIGNED") generate
				U:  daniel_ADD_SUB_UNSIGNED generic map (lpm_width => lpm_width, lpm_direction => lpm_direction)        
					 port map (dataa => dataa, datab => datab, cin => cin, add_sub => add_sub, result => result);
        end generate;

    L2: if (LPM_representation = "SIGNED") generate
				V:  daniel_ADD_SUB_SIGNED generic map (lpm_width => lpm_width, lpm_direction => lpm_direction)
                port map (dataa => dataa, datab => datab, cin => cin, add_sub => add_sub, result => result);
        end generate;

end structure;

