---------------------------------------------------------------------------
-- This VHDL file was developed by Daniel Llamocca.  It may be
-- freely copied and/or distributed at no cost.  Any persons using this
-- file for any purpose do so at their own risk, and are responsible for
-- the results of such use.  Daniel Llamocca does not guarantee that
-- this file is complete, correct, or fit for any particular purpose.
-- NO WARRANTY OF ANY KIND IS EXPRESSED OR IMPLIED.  This notice must
-- accompany any copy of this file.
--------------------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

library work;
use work.lpm_components.all;

entity sqrt_nr_x_comb is 
	generic (N: INTEGER:= 12;
				x: INTEGER:= 8;
				NQ: INTEGER:= 14); -- NQ = N/2 + x
	port (clock, resetn, s: in std_logic;
			DI	: in std_logic_vector (N - 1 downto 0); -- operand
			QO	: out std_logic_vector (NQ - 1 downto 0); -- sqrt
			done: out std_logic);
end sqrt_nr_x_comb;

architecture structure of sqrt_nr_x_comb is

	type chunk_D is array(NQ-1 downto 0) of std_logic_vector(N-1 downto 0);
	type chunk_Q is array(NQ-1 downto 0) of std_logic_vector(NQ-1 downto 0);
	type chunk_R is array(NQ-1 downto 0) of std_logic_vector(NQ downto 0);
	type chunk_T is array(NQ-1 downto 1) of std_logic_vector(NQ+1 downto 0);

	signal D: chunk_D;
	signal Q: chunk_Q;
	signal R: chunk_R;
	signal T: chunk_T;
	
	type chunk_fi is array (1 to NQ-1) of std_logic_vector (NQ-1+3 -1 downto 0);
	signal dataa_fi, datab_fi: chunk_fi;

begin

ri: lpm_ff generic map (LPM_WIDTH => N)
	 port map (data => DI, clock => clock, aclr => not(resetn), q => D(0),
				  enable => '1', aset => '0', aload => '0', sclr => '0', sset => '0', sload => '0');
	
	 D(1)(N-3 downto 0) <= D(0)(N-3 downto 0);
	 R(0)(1 downto 0) <= (D(0)(N-1) xnor D(0)(N-2)) & (not D(0)(N-2));
	 Q(0)(0) <= ((D(0)(N-1)) or (D(0)(N-2)));
	
g1: for i in 1 to (NQ-x)-1 generate	
		 fi: lpm_add_sub generic map (LPM_WIDTH => i + 3)
		     port map (dataa => dataa_fi(i)(i+3 - 1 downto 0),
					      datab => datab_fi(i)(i+3 - 1 downto 0),
						   add_sub => not (Q(i-1)(0)), aclr => '0',
						   result => T(i)((i+3)-1 downto 0));

		 dataa_fi(i)(i+3 - 1 downto 0) <= R(i-1)(i downto 0) & D(i)((N-2*i)-1 downto (N-2*i)-2);
		 datab_fi(i)(i+3 - 1 downto 0) <= '0'& Q(i-1)(i-1 downto 0)& not(Q(i-1)(0))&'1';
					
		 R(i)((i+2)-1 downto 0) <= T(i)((i+3)-2 downto 0);
		 Q(i)(i downto 0) <= Q(i-1)(i-1 downto 0)& not (T(i)((i+3)-1));

		 gg1: if i <= (NQ-x)-2 generate
				   D(i+1)(N-2*(i+1)-1 downto 0) <= D(i)(N-2*(i+1)-1 downto 0);
			   end generate;
     end generate;

g2: for i in NQ-x to NQ-1 generate -- If x=0, this 'g2' generates nothing
		 fk: lpm_add_sub generic map (LPM_WIDTH => i + 3)
		     port map (dataa => dataa_fi(i)(i+3 - 1 downto 0),
					      datab => datab_fi(i)(i+3 - 1 downto 0),
					      add_sub => not (Q(i-1)(0)), aclr => '0',
					      result => T(i)((i+3)-1 downto 0));
	 
		     dataa_fi(i)(i+3 - 1 downto 0) <= R(i-1)(i downto 0) & "00";
	        datab_fi(i)(i+3 - 1 downto 0) <= '0'& Q(i-1)(i-1 downto 0)& not(Q(i-1)(0))&'1';														
					
		     R(i)((i+2)-1 downto 0) <= T(i)((i+3)-2 downto 0);
		     Q(i)(i downto 0) <= Q(i-1)(i-1 downto 0)& not (T(i)((i+3)-1));
	 end generate;

ro: lpm_ff generic map (LPM_WIDTH => NQ)
	 port map (data => Q(NQ-1), clock => clock, aclr => not(resetn), q => QO,
			     enable => '1', aset => '0', aload => '0', sclr => '0', sset => '0', sload => '0');
				  
-- Shift register for 'done' signal
gsh: lpm_shiftreg generic map (lpm_width => 2, lpm_direction => "RIGHT")
	  port map (clock => clock, enable => '1', shiftin => s, aclr => not(resetn), shiftout => done,
	            aset => '0', sclr => '0', sset =>'0');
end structure;